@@ -12,31 +12,30 @@
 #export OS_BUILD=32
 
 #export GCC_TYPE=
+#if no override is set - use these values
+#otherwise use the ones jenkins passes thru
+#!/usr/bin/env bash
 if  [[ "${OVERRIDE}" == '' ]] ; then
-export GEOS_VER=3.8.2
-export GDAL_VER=3.4.2
-export PROJ_VER=7.2.1
-export SFCGAL_VER=1.4.1
-export PCRE_VER=8.33
-export PROTOBUF_VER=3.2.0
-export PROTOBUFC_VER=1.2.1
-export CGAL_VER=4.11
-export BOOST_VER=1.78.0
-	#BOOST_VER_WU=1_49_0
-export BOOST_VER_WU=1_78_0
-
-if [["${OS_BUILD}" == '64']] ; then
-  export GDAL_VER=2.4.4
-  export SFCGAL_VER=1.4.1
+	export GEOS_VER=3.8.1
+	export GDAL_VER=3.0.4
+	export PROJ_VER=6.3.2
+	export SFCGAL_VER=1.3.8
+	export CGAL_VER=5.0
+	export ICON_VER=1.16
+	export ZLIB_VER=1.2.11
+  export PROTOBUF_VER=3.2.0
+	export PROTOBUFC_VER=1.2.1
+	export JSON_VER=0.12
+	export PROJSO=libproj-13.dll
 fi;
 
-fi;
 export PROTOBUF_VER=3.2.0
 export PROTOBUFC_VER=1.2.1
 export JSON_VER=0.12
 export PCRE_VER=8.33
+
 if  [[ "${ICON_VER}" == '' ]] ; then
-  export ICON_VER=1.16
+  export ICON_VER=1.15
 fi;
 
 echo "ICON_VER ${ICON_VER}"
@@ -46,11 +45,6 @@ if  [[ "${ZLIB_VER}" == '' ]] ; then
   export ZLIB_VER=1.2.11
 fi;
 
-if  [[ "${BOOST_VER}" == '' ]] ; then
-  export BOOST_VER=1.78.0
-  export BOOST_VER_WU=1_78_0
-fi;
-
 
 #set to something even if override is on but not set
 if  [[ "${LIBXML_VER}" == '' ]] ; then
@@ -59,19 +53,18 @@ fi;
 
 #set to something even if override is on but not set
 if  [[ "${CGAL_VER}" == '' ]] ; then
-  export CGAL_VER=5.3
+  export CGAL_VER=5.0
 fi;
 
-if [[ "${GCC_TYPE}" == *gcc48* ]] ; then
-	export PROJECTS=/projects
-	export MINGPROJECTS=/projects
-	export PATHOLD=$PATH
-else
-	export PROJECTS=/projects
-	export MINGPROJECTS=/projects
-	export PATHOLD=$PATH
-	#export JSON_VER=0.9
-fi;
+echo "ZLIB_VER $ZLIB_VER"
+echo "PROJ_VER $PROJ_VER"
+echo "LIBXML_VER $LIBXML_VER"
+echo "CGAL_VER $CGAL_VER"
+
+export PROJECTS=/projects
+export PROJECTS=/projects
+export PATHOLD=$PATH
+
 
 export PGHOST=localhost
 
@@ -81,33 +74,21 @@ export PATHOLD=$PATH
 WEB=/home/www/postgis/htdocs
 DWN=${WEB}/download
 
-export PATHOLD="/mingw/bin:/mingw/include:/c/Windows/system32:/c/Windows:.:/bin:/include:/usr/local/bin:/c/ming${OS_BUILD}/svn"
+export PATHOLD="/mingw/bin:/mingw/include:/c/Windows/system32:/c/Windows:.:/bin:/include:/usr/local/bin"
 #export PG_VER=9.2beta2
 
 echo PATH BEFORE: $PATH
 
 export PGPATH=${PROJECTS}/postgresql/rel/pg${PG_VER}w${OS_BUILD}${GCC_TYPE}
 export PGPATHEDB=${PGPATH}edb
-export PROJSO=libproj-9.dll
+#export PROJSO=libproj-13.dll
 export POSTGIS_MINOR_VER=${POSTGIS_MAJOR_VERSION}.${POSTGIS_MINOR_VERSION}
 export POSTGIS_MICRO_VER=${POSTGIS_MAJOR_VERSION}.${POSTGIS_MINOR_VERSION}.${POSTGIS_MICRO_VERSION}
 
-if [[ "$POSTGIS_MICRO_VERSION"  == *SVN* || "$POSTGIS_MICRO_VERSION"  == *dev* ]] ; then
-	export POSTGIS_SRC=${PROJECTS}/postgis/branches/${POSTGIS_MINOR_VER}
-	export svnurl="https://svn.osgeo.org/postgis/branches/${POSTGIS_MINOR_VER}"
-else
-	#tagged version -- official release
-	export POSTGIS_SRC=${PROJECTS}/postgis/tags/${POSTGIS_MINOR_VER}.${POSTGIS_MICRO_VERSION}
-	export svnurl="https://svn.osgeo.org/postgis/tags/${POSTGIS_MINOR_VER}.${POSTGIS_MICRO_VERSION}"
-fi;
-
 if [ -n "$SOURCE_FOLDER" ]; then
   export POSTGIS_SRC=${PROJECTS}/postgis/$SOURCE_FOLDER
 fi
 
-if [[ "$reference"  == *trunk* ]] ; then
-	export svnurl="https://svn.osgeo.org/postgis/trunk"
-fi;
 #export POSTGIS_SRC=${PROJECTS}/postgis/trunk
 #POSTGIS_SVN_REVISION=will_be_passed_in_by_bot
 export GDAL_DATA="${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/share/gdal"
@@ -117,13 +98,14 @@ export REL_PGVER=${PG_VER//./} #strip the period
 
 
 export RELDIR=${PROJECTS}/postgis/builds/${POSTGIS_MINOR_VER}
-export RELVERDIR=postgis-pg${REL_PGVER}-binaries-${POSTGIS_MINOR_VER}.${POSTGIS_MICRO_VERSION}w${OS_BUILD}${GCC_TYPE}
+export RELVERDIR=postgis-pg${REL_PGVER}-binaries-${POSTGIS_MICRO_VER}w${OS_BUILD}${GCC_TYPE}
 export PATH="${PATHOLD}:${PGPATH}/bin:${PGPATH}/lib"
 export PCRE_VER=8.33
+#PATH="${PGPATH}/bin:${PGPATH}/lib:${PROJECTS}/xsltproc:${PROJECTS}/gtkw${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/gtkw${OS_BUILD}/bin:${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/include:${PROJECTS}/rel-libiconv-1.13.1w${OS_BUILD}${GCC_TYPE}/bin:${PATH}"
+#echo PATH AFTER: $PATH
 outdir="${RELDIR}/${RELVERDIR}"
 package="${RELDIR}/${RELVERDIR}.zip"
 verfile="${RELDIR}/${RELVERDIR}/version.txt"
-#svnurl="http://svn.osgeo.org/postgis/trunk"
 rm -rf $outdir
 rm $package
 mkdir -p $outdir
@@ -136,19 +118,21 @@ mkdir $outdir/bin/postgisgui
 mkdir $outdir/bin/postgisgui/share
 mkdir $outdir/bin/postgisgui/lib
 mkdir $outdir/utils
-cp ${MINGPROJECTS}/rel-libiconv-${ICONV_VER}w${OS_BUILD}/bin/*.dll  $outdir/bin/postgisgui
+cp ${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll  $outdir/bin/postgisgui
 # it seems 9.2 and 9.3 doesn't come with its own libiconv good grief
 # and trying to use their libiconv2.dll makes shp2pgsql crash
 if [[ "$PG_VER" == *9.2* || "$PG_VER" == *9.3* ]]; then
-	cp ${MINGPROJECTS}/rel-libiconv-${ICONV_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll  $outdir/bin
+	cp ${PROJECTS}/rel-libiconv-1.13.1w${OS_BUILD}${GCC_TYPE}/bin/*.dll  $outdir/bin
 fi;
 cp ${PGPATHEDB}/bin/libpq.dll  $outdir/bin/postgisgui
 #cp ${PGPATHEDB}/bin/libiconv2.dll  $outdir/bin/postgisgui
-cp ${MINGPROJECTS}/rel-libiconv-${ICONV_VER}w${OS_BUILD}${GCC_TYPE}/bin/libicon*.dll $outdir/bin/postgisgui
+cp ${PROJECTS}/rel-libiconv-${ICON_VER}w${OS_BUILD}${GCC_TYPE}/bin/libicon*.dll $outdir/bin/postgisgui
 cp ${PGPATHEDB}/bin/libintl*.dll $outdir/bin/postgisgui
 
 cp ${PGPATHEDB}/bin/ssleay32.dll $outdir/bin/postgisgui
 cp ${PGPATHEDB}/bin/libeay32.dll $outdir/bin/postgisgui
+cp ${PGPATHEDB}/bin/libcrypto-1_1-x64.dll $outdir/bin/postgisgui
+cp ${PGPATHEDB}/bin/libssl-1_1-x64.dll $outdir/bin/postgisgui
 
 cp /c/ming${OS_BUILD}${GCC_TYPE}/mingw${OS_BUILD}/bin/libstdc++-6.dll $outdir/bin
 cp /c/ming${OS_BUILD}${GCC_TYPE}/mingw${OS_BUILD}/bin/libgcc*.dll $outdir/bin
@@ -162,19 +146,19 @@ cp -r ${PROJECTS}/gtkw${OS_BUILD}${GCC_TYPE}/lib/*.dll $outdir/bin/postgisgui/li
 cp -r ${PROJECTS}/gtkw${OS_BUILD}${GCC_TYPE}/lib/gdk-pixbuf-2.0 $outdir/bin/postgisgui/lib
 
 
-cp ${MINGPROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/share/proj/* $outdir/share/contrib/postgis-${POSTGIS_MINOR_VER}/proj
-cp ${MINGPROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll $outdir/bin
+cp ${PROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/share/proj/* $outdir/share/contrib/postgis-${POSTGIS_MINOR_VER}/proj
+cp ${PROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll $outdir/bin
 cp -p ${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll $outdir/bin
 
-cp ${MINGPROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll $outdir/bin/postgisgui
+cp ${PROJECTS}/proj/rel-${PROJ_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll $outdir/bin/postgisgui
 cp -p ${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll $outdir/bin/postgisgui
 
 #for protobuf
 cp ${PROJECTS}/protobuf/rel-${PROTOBUF_VER}w${OS_BUILD}${GCC_TYPE}/bin/libprotobuf-c-*.dll $outdir/bin
 
-echo "POSTGIS: ${POSTGIS_MICRO_VER} https://postgis.net/source" > $verfile
+echo "POSTGIS: ${POSTGIS_MINOR_VER} r${POSTGIS_SVN_REVISION} http://postgis.net/source" > $verfile
 
-if [ "$POSTGIS_MAJOR_VERSION" == "2" ] ; then
+if [ "$POSTGIS_MAJOR_VERSION" > "1" ] ; then
   ## only copy gdal components if 2+.  1.5 doesn't have raster support
   cp -p ${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll $outdir/bin
   cp -rp  ${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/share/gdal $outdir/gdal-data
@@ -189,8 +173,8 @@ fi;
 
 if [ -n "$SFCGAL_VER"  ]; then
 	## only copy cgal and sfcgal stuff if sfcgal is packaged
-	export BOOST_VER=1.78.0
-	export BOOST_VER_WU=1_78_0
+	export BOOST_VER=1.59.0
+	export BOOST_VER_WU=1_59_0
 	export GMP_VER=5.1.2
 	export MPFR_VER=3.1.2
 	echo "CGAL VERSION: ${CGAL_VER} http://www.cgal.org" >> $verfile
@@ -206,35 +190,38 @@ fi;
 
 echo "PROTOBUF VERSION: ${PROTOBUF_VER} https://github.com/google/protobuf" >> $verfile
 echo "PROTOBUF-C VERSION: ${PROTOBUFC_VER} https://github.com/protobuf-c/protobuf-c"  >> $verfile
-#cp ${MINGPROJECTS}/libxml/rel-libxml2-2.7.8w${OS_BUILD}/bin/*.dll  $outdir/bin/
-cp ${PGPATHEDB}/bin/libxml2-2.dll   $outdir/bin/
+cp ${PROJECTS}/libxml/rel-libxml2-${LIBXML_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll  $outdir/bin/
+#cp ${PGPATHEDB}/bin/libxml2-2.dll   $outdir/bin/
 
 cd ${POSTGIS_SRC}
-strip postgis/postgis-${POSTGIS_MINOR_VER}.dll
-strip raster/rt_pg/rtpostgis-${POSTGIS_MINOR_VER}.dll
+strip postgis/*.dll
+strip sfcgal/*.dll
+strip raster/rt_pg/*.dll
 strip liblwgeom/.libs/*.dll
 
-cp postgis/postgis-${POSTGIS_MINOR_VER}.dll ${RELDIR}/${RELVERDIR}/lib
+cp postgis/*.dll ${RELDIR}/${RELVERDIR}/lib
+cp sfcgal/*.dll ${RELDIR}/${RELVERDIR}/lib
 cp topology/*.dll ${RELDIR}/${RELVERDIR}/lib
-cp raster/rt_pg/rtpostgis-${POSTGIS_MINOR_VER}.dll ${RELDIR}/${RELVERDIR}/lib
+cp raster/rt_pg/*.dll ${RELDIR}/${RELVERDIR}/lib
 cp doc/*_comments.sql ${RELDIR}/${RELVERDIR}/share/contrib/postgis-${POSTGIS_MINOR_VER}
 cp postgis/*.sql ${RELDIR}/${RELVERDIR}/share/contrib/postgis-${POSTGIS_MINOR_VER}
 cp raster/rt_pg/*.sql ${RELDIR}/${RELVERDIR}/share/contrib/postgis-${POSTGIS_MINOR_VER}
 cp utils/*.pl ${RELDIR}/${RELVERDIR}/utils
-#add extras
-svn export "${svnurl}/extras" ${RELDIR}/${RELVERDIR}/share/contrib/postgis-${POSTGIS_MINOR_VER}/extras
-#cp raster/rt_pg/rtpostgis.sql ${RELDIR}/${RELVERDIR}/share/contrib/postgis-${POSTGIS_MINOR_VER}
-#cp raster/rt_pg/rtpostgis_upgrade_20_minor.sql ${RELDIR}/${RELVERDIR}/share/contrib/postgis-${POSTGIS_MINOR_VER}
 
 cp raster/loader/.libs/raster2pgsql.exe ${RELDIR}/${RELVERDIR}/bin
+cp raster/loader/raster2pgsql.exe ${RELDIR}/${RELVERDIR}/bin
 cp liblwgeom/.libs/*.dll ${RELDIR}/${RELVERDIR}/bin
+cp loader/shp2pgsql.exe ${RELDIR}/${RELVERDIR}/bin
 cp loader/.libs/shp2pgsql.exe ${RELDIR}/${RELVERDIR}/bin
+cp loader/pgsql2shp.exe ${RELDIR}/${RELVERDIR}/bin
 cp loader/.libs/pgsql2shp.exe ${RELDIR}/${RELVERDIR}/bin
+cp loader/shp2pgsql-gui.exe ${RELDIR}/${RELVERDIR}/bin/postgisgui
 cp loader/.libs/shp2pgsql-gui.exe ${RELDIR}/${RELVERDIR}/bin/postgisgui
+
 #cp liblwgeom/.libs/*.dll ${RELDIR}/${RELVERDIR}/bin/postgisgui
 
 #shp2pgsql-gui now has dependency on geos (though in theory it shouldn't)
-cp -p ${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}/bin/*.dll ${RELDIR}/${RELVERDIR}/bin/postgisgui
+cp -p ${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll ${RELDIR}/${RELVERDIR}/bin/postgisgui
 cp spatial_ref_sys.sql ${RELDIR}/${RELVERDIR}/share/contrib/postgis-${POSTGIS_MINOR_VER}
 cp topology/topology.sql ${RELDIR}/${RELVERDIR}/share/contrib/postgis-${POSTGIS_MINOR_VER}
 cp topology/topology_upgrade_*.sql ${RELDIR}/${RELVERDIR}/share/contrib/postgis-${POSTGIS_MINOR_VER}
@@ -251,13 +238,21 @@ cp -r extensions/*/*.dll ${RELDIR}/${RELVERDIR}/lib #only address_standardizer i
 cp -r ${RELDIR}/packaging_notes/* ${RELDIR}/${RELVERDIR}/
 
 
-echo "GEOS VERSION: ${GEOS_VER} https://libgeos.org/usage/download/" >> $verfile
-echo "GDAL VERSION: ${GDAL_VER} https://proj.org/download.html" >> $verfile
-echo "PROJ VERSION: ${PROJ_VER} https://proj.org/download.html" >> $verfile
+echo "GEOS VERSION: ${GEOS_VER} https://trac.osgeo.org/geos" >> $verfile
+echo "GDAL VERSION: ${GDAL_VER} https://gdal.org/download.html#current-releases" >> $verfile
+echo "PROJ VERSION: ${PROJ_VER} https://proj.org/download.html#current-release" >> $verfile
+
+echo "LIBICONV VERSION: ${ICON_VER} http://ftp.gnu.org/gnu/libiconv/libiconv-${ICONV_VER}.tar.gz" >> $verfile
 
 if [ -n "$SFCGAL_VER"  ]; then
-    echo "SFCGAL VERSION: ${SFCGAL_VER} http://www.sfcgal.org https://gitlab.com/Oslandia/SFCGAL" >> $verfile
+    echo "CGAL VERSION: ${CGAL_VER} http://www.cgal.org" >> $verfile
+    echo "BOOST VERSION: ${BOOST_VER} http://www.boost.org" >> $verfile
+    echo "SFCGAL VERSION: ${SFCGAL_VER} http://www.sfcgal.org https://github.com/Oslandia/SFCGAL" >> $verfile
 fi;
+
+if [ -f ${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/gdal_depends.txt ]; then
+	cat $verfile ${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}${GCC_TYPE}/gdal_depends.txt > $verfile
+fi
 #echo "PAGC ADDRESS STANDARDIZER: http://sourceforge.net/p/pagc/code/HEAD/tree/branches/sew-refactor/postgresql " >> $verfile
 cd ${RELDIR}
 zip -r $package ${RELVERDIR}
