@@ -166,24 +166,26 @@ colmap_init(colmap *map)
 void
 colmap_clean(colmap *map)
 {
-  int i;
-  if (map->size)
-  {
-    for (i = 0; i < map->size; i++)
-    {
-      if (map->pgfieldnames[i]) free(map->pgfieldnames[i]);
-      if (map->dbffieldnames[i]) free(map->dbffieldnames[i]);
-    }
-    free(map->pgfieldnames);
-    free(map->dbffieldnames);
-  }
+	int i;
+	if (map != NULL){
+		if (map->size)
+		{
+			for (i = 0; i < map->size; i++)
+			{
+				if (map->pgfieldnames[i]) free(map->pgfieldnames[i]);
+				if (map->dbffieldnames[i]) free(map->dbffieldnames[i]);
+			}
+			free(map->pgfieldnames);
+			free(map->dbffieldnames);
+		}
+	}
 }
 
 const char *
 colmap_dbf_by_pg(colmap *map, const char *pgname)
 {
   int i;
-  for (i=0; i<map->size; ++i)
+  for (i=0; i<map->size; i++)
   {
     if (!strcasecmp(map->pgfieldnames[i], pgname))
     {
@@ -197,7 +199,7 @@ const char *
 colmap_pg_by_dbf(colmap *map, const char *dbfname)
 {
   int i;
-  for (i=0; i<map->size; ++i)
+  for (i=0; i<map->size; i++)
   {
     if (!strcasecmp(map->dbffieldnames[i], dbfname))
     {
