@@ -35,8 +35,7 @@
 Datum transform(PG_FUNCTION_ARGS);
 Datum transform_geom(PG_FUNCTION_ARGS);
 Datum postgis_proj_version(PG_FUNCTION_ARGS);
-
-
+Datum LWGEOM_asKML(PG_FUNCTION_ARGS);
 
 /**
  * transform( GEOMETRY, INT (output srid) )
@@ -47,44 +46,45 @@ Datum postgis_proj_version(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(transform);
 Datum transform(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom;
-	GSERIALIZED *result=NULL;
-	LWGEOM *lwgeom;
-	projPJ input_pj, output_pj;
-	int32 output_srid, input_srid;
+	GSERIALIZED* geom;
+	GSERIALIZED* result=NULL;
+	LWGEOM* lwgeom;
+	LWPROJ *pj;
+	int32 srid_to, srid_from;
 
-	output_srid = PG_GETARG_INT32(1);
-	if (output_srid == SRID_UNKNOWN)
+	srid_to = PG_GETARG_INT32(1);
+	if (srid_to == SRID_UNKNOWN)
 	{
-		elog(ERROR, "%d is an invalid target SRID", SRID_UNKNOWN);
+		elog(ERROR, "ST_Transform: %d is an invalid target SRID", SRID_UNKNOWN);
 		PG_RETURN_NULL();
 	}
 
 	geom = PG_GETARG_GSERIALIZED_P_COPY(0);
-	input_srid = gserialized_get_srid(geom);
+	srid_from = gserialized_get_srid(geom);
 
-	if ( input_srid == SRID_UNKNOWN )
+	if ( srid_from == SRID_UNKNOWN )
 	{
 		PG_FREE_IF_COPY(geom, 0);
-		elog(ERROR, "Input geometry has unknown (%d) SRID", SRID_UNKNOWN);
+		elog(ERROR, "ST_Transform: Input geometry has unknown (%d) SRID", SRID_UNKNOWN);
 		PG_RETURN_NULL();
 	}
 
 	/* Input SRID and output SRID are equal, noop */
-	if ( input_srid == output_srid )
+	if ( srid_from == srid_to )
 		PG_RETURN_POINTER(geom);
 
-	if ( GetProjectionsUsingFCInfo(fcinfo, input_srid, output_srid, &input_pj, &output_pj) == LW_FAILURE )
+	postgis_initialize_cache();
+	if ( GetLWPROJ(srid_from, srid_to, &pj) == LW_FAILURE )
 	{
 		PG_FREE_IF_COPY(geom, 0);
-		elog(ERROR, "Failure reading projections from spatial_ref_sys.");
+		elog(ERROR, "ST_Transform: Failure reading projections from spatial_ref_sys.");
 		PG_RETURN_NULL();
 	}
 
 	/* now we have a geometry, and input/output PJ structs. */
 	lwgeom = lwgeom_from_gserialized(geom);
-	lwgeom_transform(lwgeom, input_pj, output_pj);
-	lwgeom->srid = output_srid;
+	lwgeom_transform(lwgeom, pj);
+	lwgeom->srid = srid_to;
 
 	/* Re-compute bbox if input had one (COMPUTE_BBOX TAINTING) */
 	if ( lwgeom->bbox )
@@ -110,96 +110,118 @@ Datum transform(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(transform_geom);
 Datum transform_geom(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom;
-	GSERIALIZED *result=NULL;
-	LWGEOM *lwgeom;
-	projPJ input_pj, output_pj;
-	char *input_proj4, *output_proj4;
-	text *input_proj4_text;
-	text *output_proj4_text;
-	int32 result_srid ;
-	char *pj_errstr;
+	GSERIALIZED *gser, *gser_result=NULL;
+	LWGEOM *geom;
+	char *input_srs, *output_srs;
+	int32 result_srid;
+	int rv;
 
-	result_srid = PG_GETARG_INT32(3);
-	geom = PG_GETARG_GSERIALIZED_P_COPY(0);
-
-	/* Set the search path if we haven't already */
-	SetPROJ4LibPath();
-
-	/* Read the arguments */
-	input_proj4_text  = (PG_GETARG_TEXT_P(1));
-	output_proj4_text = (PG_GETARG_TEXT_P(2));
+	/* Take a copy, since we will be altering the coordinates */
+	gser = PG_GETARG_GSERIALIZED_P_COPY(0);
 
 	/* Convert from text to cstring for libproj */
-	input_proj4 = text_to_cstring(input_proj4_text);
-	output_proj4 = text_to_cstring(output_proj4_text);
-
-	/* make input and output projection objects */
-	input_pj = lwproj_from_string(input_proj4);
-	if ( input_pj == NULL )
-	{
-		pj_errstr = pj_strerrno(*pj_get_errno_ref());
-		if ( ! pj_errstr ) pj_errstr = "";
-
-		/* we need this for error reporting */
-		/* pfree(input_proj4); */
-		pfree(output_proj4);
-		pfree(geom);
-
-		elog(ERROR,
-		    "transform_geom: could not parse proj4 string '%s' %s",
-		    input_proj4, pj_errstr);
-		PG_RETURN_NULL();
-	}
-	pfree(input_proj4);
-
-	output_pj = lwproj_from_string(output_proj4);
-
-	if ( output_pj == NULL )
-	{
-		pj_errstr = pj_strerrno(*pj_get_errno_ref());
-		if ( ! pj_errstr ) pj_errstr = "";
-
-		/* we need this for error reporting */
-		/* pfree(output_proj4); */
-		pj_free(input_pj);
-		pfree(geom);
-
-		elog(ERROR,
-			"transform_geom: couldn't parse proj4 output string: '%s': %s",
-			output_proj4, pj_errstr);
-		PG_RETURN_NULL();
-	}
-	pfree(output_proj4);
+	input_srs = text_to_cstring(PG_GETARG_TEXT_P(1));
+	output_srs = text_to_cstring(PG_GETARG_TEXT_P(2));
+	result_srid = PG_GETARG_INT32(3);
 
 	/* now we have a geometry, and input/output PJ structs. */
-	lwgeom = lwgeom_from_gserialized(geom);
-	lwgeom_transform(lwgeom, input_pj, output_pj);
-	lwgeom->srid = result_srid;
+	geom = lwgeom_from_gserialized(gser);
+	rv = lwgeom_transform_from_str(geom, input_srs, output_srs);
+	pfree(input_srs);
+	pfree(output_srs);
 
-	/* clean up */
-	pj_free(input_pj);
-	pj_free(output_pj);
-
-	/* Re-compute bbox if input had one (COMPUTE_BBOX TAINTING) */
-	if ( lwgeom->bbox )
+	if (rv == LW_FAILURE)
 	{
-		lwgeom_refresh_bbox(lwgeom);
+		elog(ERROR, "coordinate transformation failed");
+		PG_RETURN_NULL();
 	}
 
-	result = geometry_serialize(lwgeom);
+	/* Re-compute bbox if input had one (COMPUTE_BBOX TAINTING) */
+	geom->srid = result_srid;
+	if (geom->bbox)
+		lwgeom_refresh_bbox(geom);
 
-	lwgeom_free(lwgeom);
-	PG_FREE_IF_COPY(geom, 0);
+	gser_result = geometry_serialize(geom);
+	lwgeom_free(geom);
+	PG_FREE_IF_COPY(gser, 0);
 
-	PG_RETURN_POINTER(result); /* new geometry */
+	PG_RETURN_POINTER(gser_result); /* new geometry */
 }
 
 
 PG_FUNCTION_INFO_V1(postgis_proj_version);
 Datum postgis_proj_version(PG_FUNCTION_ARGS)
 {
+#if POSTGIS_PROJ_VERSION < 60
 	const char *ver = pj_get_release();
 	text *result = cstring_to_text(ver);
+#else
+	PJ_INFO pji = proj_info();
+	text *result = 	cstring_to_text(pji.version);
+#endif
 	PG_RETURN_POINTER(result);
 }
+
+
+/**
+ * Encode feature in KML
+ */
+PG_FUNCTION_INFO_V1(LWGEOM_asKML);
+Datum LWGEOM_asKML(PG_FUNCTION_ARGS)
+{
+	LWGEOM *lwgeom;
+	lwvarlena_t *kml;
+	const char *default_prefix = ""; /* default prefix */
+	char *prefixbuf;
+	const char *prefix = default_prefix;
+	int32_t srid_from;
+	const int32_t srid_to = 4326;
+
+	/* Get the geometry */
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P_COPY(0);
+	int precision = PG_GETARG_INT32(1);
+	text *prefix_text = PG_GETARG_TEXT_P(2);
+	srid_from = gserialized_get_srid(geom);
+
+	if ( srid_from == SRID_UNKNOWN )
+	{
+		PG_FREE_IF_COPY(geom, 0);
+		elog(ERROR, "ST_AsKML: Input geometry has unknown (%d) SRID", SRID_UNKNOWN);
+		PG_RETURN_NULL();
+	}
+
+	/* Condition precision */
+	if (precision < 0)
+		precision = 0;
+
+	if (VARSIZE_ANY_EXHDR(prefix_text) > 0)
+	{
+		/* +2 is one for the ':' and one for term null */
+		prefixbuf = palloc(VARSIZE_ANY_EXHDR(prefix_text)+2);
+		memcpy(prefixbuf, VARDATA(prefix_text),
+		       VARSIZE_ANY_EXHDR(prefix_text));
+		/* add colon and null terminate */
+		prefixbuf[VARSIZE_ANY_EXHDR(prefix_text)] = ':';
+		prefixbuf[VARSIZE_ANY_EXHDR(prefix_text)+1] = '\0';
+		prefix = prefixbuf;
+	}
+
+	lwgeom = lwgeom_from_gserialized(geom);
+
+	if (srid_from != srid_to)
+	{
+		LWPROJ *pj;
+		if (GetLWPROJ(srid_from, srid_to, &pj) == LW_FAILURE)
+		{
+			PG_FREE_IF_COPY(geom, 0);
+			elog(ERROR, "ST_AsKML: Failure reading projections from spatial_ref_sys.");
+			PG_RETURN_NULL();
+		}
+		lwgeom_transform(lwgeom, pj);
+	}
+
+	kml = lwgeom_to_kml2(lwgeom, precision, prefix);
+	if (kml)
+		PG_RETURN_TEXT_P(kml);
+	PG_RETURN_NULL();
+}
