@@ -34,7 +34,10 @@
 #include "lwgeom_log.h"
 #include "lwgeom_pg.h"
 #include "geos_c.h"
-#include "lwgeom_backend_api.h"
+
+#ifdef HAVE_WAGYU
+#include "lwgeom_wagyu.h"
+#endif
 
 /*
  * This is required for builds against pgsql
@@ -58,7 +61,6 @@ void _PG_init(void);
 void
 _PG_init(void)
 {
-
   coreIntHandler = pqsignal(SIGINT, handleInterrupt);
 
 #ifdef WIN32
@@ -66,11 +68,8 @@ _PG_init(void)
   lwgeom_register_interrupt_callback(interruptCallback);
 #endif
 
-    /* install PostgreSQL handlers */
-    pg_install_lwgeom_handlers();
-
-    /* initialize geometry backend */
-    lwgeom_init_backend();
+  /* install PostgreSQL handlers */
+  pg_install_lwgeom_handlers();
 }
 
 /*
@@ -97,6 +96,10 @@ handleInterrupt(int sig)
 
   GEOS_interruptRequest();
 
+#ifdef HAVE_WAGYU
+  lwgeom_wagyu_interruptRequest();
+#endif
+
   /* request interruption of liblwgeom as well */
   lwgeom_request_interrupt();
 
