@@ -78,7 +78,7 @@ rt_pixtype_alignment(rt_pixtype pixtype) {
 
 rt_pixtype
 rt_pixtype_index_from_name(const char* pixname) {
-	assert(pixname && strlen(pixname) > 0);
+	assert(pixname);
 
 	if (strcmp(pixname, "1BB") == 0)
 		return PT_1BB;
@@ -393,7 +393,8 @@ rt_errorstate rt_pixel_set_to_array(
 			/* unweighted (boolean) mask */
 			if (mask->weighted == 0) {
 				/* pixel is set to zero or nodata */
-				if (FLT_EQ(mask->values[_y][_x],0) || mask->nodata[_y][_x] == 1) {
+				if (FLT_EQ(mask->values[_y][_x], 0.0) || mask->nodata[_y][_x] == 1)
+				{
 					values[_y][_x] = 0;
 					nodatas[_y][_x] = 1;
 				}
