@@ -87,7 +87,7 @@ inp as ( select
 )'
  ::geometry as g),
 tg as ( select totopogeom(g, 'tt', 5) as g from inp )
-select St_AsText(inp.g), st_astext(ST_Normalize(tg.g::geometry) ) from inp, tg;
+select St_AsText(inp.g), st_astext(ST_Normalize(tg.g::geometry)) from inp, tg;
 
 -- Convert some empties
 SELECT ST_AsText(toTopoGeom('POINT EMPTY', 'tt', 1)::geometry);
@@ -199,6 +199,133 @@ SELECT 'tgup1.3', id(t.g), st_area(t.g), count(r.*)
   WHERE t.id = -1 AND r.layer_id = 4 AND r.topogeo_id = id(t.g)
   GROUP BY id(t.g), st_area(t.g);
 
+-- Check GeometryType of TopoGeometries getting
+-- other TopoGeometry types added
+-- See https://trac.osgeo.org/postgis/ticket/4854
+BEGIN;
+SELECT '#4854.0.0', GeometryType(
+    toTopoGeom(
+      'POINT(0 0)', -- insert a point
+      toTopoGeom(
+        'POINT(10 0)', -- to a point TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.0.1', GeometryType(
+    toTopoGeom(
+      'POINT(0 0)', -- insert a point
+      toTopoGeom(
+        'LINESTRING(0 0, 10 0)', -- to a linear TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.0.2', GeometryType(
+    toTopoGeom(
+      'POINT(0 0)', -- insert a point
+      toTopoGeom(
+        'POLYGON((0 0,10 0,10 10,0 10,0 0))', -- to a polygonal TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.0.3', GeometryType(
+    toTopoGeom(
+      'POINT(0 0)', -- insert a point
+      toTopoGeom(
+        'GEOMETRYCOLLECTION(POLYGON((0 0,10 0,10 10,0 10,0 0)),POINT(5 5))', -- to a collection TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.1.0', GeometryType(
+    toTopoGeom(
+      'LINESTRING(10 0, 20 0)', -- insert a line
+      toTopoGeom(
+        'POINT(0 0)', -- to a point TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.1.1', GeometryType(
+    toTopoGeom(
+      'LINESTRING(10 0, 20 0)', -- insert a line
+      toTopoGeom(
+        'LINESTRING(0 0, 0 10)', -- to a line TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.1.2', GeometryType(
+    toTopoGeom(
+      'LINESTRING(10 0, 20 0)', -- insert a line
+      toTopoGeom(
+        'POLYGON((0 0, 10 0,10 10,0 10,0 0))', -- to a polygonal TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.1.3', GeometryType(
+    toTopoGeom(
+      'LINESTRING(10 0, 20 0)', -- insert a line
+      toTopoGeom(
+        'GEOMETRYCOLLECTION(POINT(0 0),LINESTRING(0 0, 10 0))', -- to a collection TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.2.0', GeometryType(
+    toTopoGeom(
+      'POLYGON((100 0,100 10,110 10,110 0, 100 0))', -- insert a polygon
+      toTopoGeom(
+        'POINT(100 1)', -- to a puntal TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.2.1', GeometryType(
+    toTopoGeom(
+      'POLYGON((-1 0,0 -2,1 0,-1 0))', -- insert a polygon
+      toTopoGeom(
+        'LINESTRING(200 1, 200 2)', -- to a lineal TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.2.2', GeometryType(
+    toTopoGeom(
+      'POLYGON((-1 0,0 -2,1 0,-1 0))', -- insert a polygon
+      toTopoGeom(
+        'POLYGON((200 1, 200 5,210 2,200 1))', -- to a polygon TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+SELECT '#4854.2.3', GeometryType(
+    toTopoGeom(
+      'POLYGON((-1 0,0 -2,1 0,-1 0))', -- insert a polygon
+      toTopoGeom(
+        'GEOMETRYCOLLECTION(POINT(0 0),LINESTRING(0 0,10 0))', -- to a collection TopoGeom
+        'tt',
+        5
+      ),
+      5)
+  );
+ROLLBACK;
+
+--------------------------------------------------------
 -- http://trac.osgeo.org/postgis/ticket/3359
 -- NOTE: requires identifier of the second edge to be 2
 TRUNCATE tt.relation CASCADE;
